package org.bouncycastle.asn1.x509;

import java.io.*;
import java.util.Enumeration;
import java.math.BigInteger;

import org.bouncycastle.asn1.*;
import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;

/**
 * The object that contains the public key stored in a certficate.
 * <p>
 * The getEncoded() method in the public keys in the JCE produces a DER
 * encoded one of these.
 */
public class SubjectPublicKeyInfo
    implements DEREncodable
{
    private AlgorithmIdentifier     algId;
    private DERObject               pubKey;

    public SubjectPublicKeyInfo(
        AlgorithmIdentifier algId,
        DERObject           publicKey)
    {
        this.pubKey = publicKey;
        this.algId = algId;
    }

    public SubjectPublicKeyInfo(
        DERConstructedSequence  seq)
    {
        Enumeration         e = seq.getObjects();

        algId = new AlgorithmIdentifier((DERConstructedSequence)e.nextElement());

        byte[]  keyData = ((DERBitString)e.nextElement()).getBytes();

        try
        {
            ByteArrayInputStream    bIn = new ByteArrayInputStream(keyData);
            DERInputStream          dIn = new DERInputStream(bIn);

            pubKey = (DERObject)dIn.readObject();
        }
        catch (IOException ex)
        {
            throw new IllegalArgumentException("error recovering public key");
        }
    }

    public AlgorithmIdentifier getAlgorithmId()
    {
        return algId;
    }

    public DERObject getPublicKey()
    {
        return pubKey;
    }

    /**
     * <pre>
     * SubjectPublicKeyInfo ::= SEQUENCE {
     *                          algorithm AlgorithmIdentifier,
     *                          publicKey BIT STRING }
     * </pre>
     */
    public DERObject getDERObject()
    {
        DERConstructedSequence  seq = new DERConstructedSequence();

        seq.addObject(algId);
        seq.addObject(new DERBitString(pubKey));

        return seq;
    }
}
